#ifndef __BOUND_H__
#define __BOUND_H__

#include "point.h"
#include "matrix.h"
#include "datalib.h"

//////////////////////////////////////////////////////////////////////////////
// CBound Class
//

class CBound
{
//////////////////////////////////////////////////
// Enumerated Types
//
public:
enum BoundIntersectionType {bitDisjoint, bitEnclosed, bitIntersect, bitEnveloping,
			    bitSize};
enum BoundType {bbtMin, bbtMax, bbtSize};

//////////////////////////////////////////////////
// Initialization
//
public:
  CBound();
  ~CBound();

//////////////////////////////////////////////////
// Interface
//
public:
  VOID Null();
////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: These are the methods you will be dealing with
// 
  // Insert a point and grow the bounding box
  VOID Insert(const Point &pt);
  // Insert a bounding box and grow the bounding box
  VOID Insert(const CBound &bound);
  // Tranform and insert a bounding box and grow the bounding box
  VOID Insert(const Matrix &m, const CBound &bound); 
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////

public:
  inline BOOL GetEmpty() const;
  inline const Point &Get(BoundType bbt) const;
  inline Point GetCenter();
  inline FLOAT GetRadius();

//////////////////////////////////////////////////
// Rendering
//
public:
////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: These are the methods you will be dealing with
// 
  VOID Render(const Matrix &m);
  VOID Render(const Matrix &mGvp_proj, const Matrix &mGproj_obj);
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Debugging
//
public:
  VOID Display();


//////////////////////////////////////////////////
// Class Methods
//
public:

//////////////////////////////////////////////////
// Class Variables
//
protected:

//////////////////////////////////////////////////
// Member Variables
//
protected:
  BOOL m_bEmpty;
  Point m_pptMinMax[bbtSize];
};

//////////////////////////////////////////////////////////////////////
// External Operators
//
inline BOOL operator==(CBound &bound0, CBound &bound1);
inline BOOL operator!=(CBound &bound0, CBound &bound1);

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline BOOL CBound::GetEmpty() const
{
  return m_bEmpty;
}

const Point &CBound::Get(BoundType bbt) const
{
  ASSERT( bbt < bbtSize );
  
  return m_pptMinMax[bbt];
}

Point CBound::GetCenter()
{
  return m_pptMinMax[bbtMin] + ((m_pptMinMax[bbtMax] - m_pptMinMax[bbtMin])*0.5);
}

FLOAT CBound::GetRadius()
{
  return ((m_pptMinMax[bbtMax] - m_pptMinMax[bbtMin]).Magnitude()) * (FLOAT)0.5;
}

inline BOOL operator==(CBound &bound0, CBound &bound1)
{
  return ( (bound0.Get(CBound::bbtMin) == 
	    bound1.Get(CBound::bbtMin)) &&
	   (bound0.Get(CBound::bbtMax) == 
	    bound1.Get(CBound::bbtMax)) );
}

inline BOOL operator!=(CBound &bound0, CBound &bound1)
{
  return !(bound0 == bound1);
}

#endif // __BOUND_H__
