#include "slideinstance.h"

#include "slideworld.h"
#include "slidetransform.h"
#include "slidesurface.h"

//////////////////////////////////////////////////////////////////////
// CSLIDEInstance Class
//

CSLIDEInstance::CSLIDEInstance()
{
  m_pcChildID = NULL;
  m_pCSLIDENode = NULL;
  m_pcSurfaceID = NULL;
  m_pCSLIDESurface = NULL;
}

CSLIDEInstance::~CSLIDEInstance()
{
}

VOID CSLIDEInstance::Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			  CHAR *pcChildID)
{
  CSLIDENode::Init(pcFile, uiLine, uiColumn, NULL);

  ASSERT( pcChildID != NULL );

  m_pcChildID = pcChildID;
  m_pCSLIDENode = NULL;
  m_pcSurfaceID = gcppcSLIDEReservedWords[sliderwtSLF_INHERIT];
  m_pCSLIDESurface = NULL;
  m_dyniShading.SetValue(slideshtSLF_INHERIT);
  m_dyniLOD.SetValue(slidelodtSLF_FULL);
  
  m_listCSLIDETransform.Init();
  m_mTransform.SetIdentity();
  m_mInverse.SetIdentity();
}

VOID CSLIDEInstance::Uninit()
{
  CSLIDENode::Uninit();
}

BOOL CSLIDEInstance::Initialized()
{
  if ( !CSLIDENode::Initialized() )
    {
      return FALSE;
    }

  return TRUE;
}

VOID CSLIDEInstance::InsertTransform(CSLIDETransform *pCSLIDETransform)
{
  ASSERT( pCSLIDETransform != NULL );

  m_listCSLIDETransform.InsertLast(pCSLIDETransform, 0);
}

BOOL CSLIDEInstance::CheckSemantics(CSLIDEWorld *pCSLIDEWorld)
{
  if ( !CSLIDENode::CheckSemantics(pCSLIDEWorld) )
    {
      return FALSE;
    }

  m_pCSLIDESurface = pCSLIDEWorld->PeekSurface(m_pcSurfaceID);
  if ( m_pCSLIDESurface == NULL )
    {
      HandleError("Undefined %s \"%s\"", 
		  gcppcSLIDEReservedWords[sliderwtSurface], 
		  m_pcSurfaceID);
      return FALSE;
    }

  ASSERT( 0 <= m_dyniShading.GetValue() &&
	  m_dyniShading.GetValue() < slideshtSize );
  ASSERT( 0 <= m_dyniLOD.GetValue() &&
	  m_dyniLOD.GetValue() < slidelodtSize );

  m_pCSLIDENode = pCSLIDEWorld->PeekNode(m_pcChildID);
  if ( m_pCSLIDENode == NULL )
    {
      HandleError("Undefined %s \"%s\"", 
		  gcppcSLIDEReservedWords[sliderwtObject], m_pcChildID);
      return FALSE;
    }

  return TRUE;
}

BOOL CSLIDEInstance::Valid()
{
  if ( !CSLIDENode::Valid() )
    {
      return FALSE;
    }

  return TRUE;
}

CSLIDEPrimitiveIter *CSLIDEInstance::AllocateIter()
{
  CSLIDEInstanceIter *pCSLIDEInstanceIter;

  pCSLIDEInstanceIter = new CSLIDEInstanceIter;
  ASSERT(pCSLIDEInstanceIter != NULL );
  pCSLIDEInstanceIter->Init(this);

  return pCSLIDEInstanceIter;
}

VOID CSLIDEInstance::FreeIter(CSLIDEPrimitiveIter *pCSLIDEInstanceIter)
{
  delete pCSLIDEInstanceIter;
}

CSLIDECamera *CSLIDEInstance::FindCamera(CListIterInt *piterIDs, 
					 CListInt *plistCSLIDEInstance)
{
  ASSERT( piterIDs != NULL );
  ASSERT( plistCSLIDEInstance != NULL );

  return m_pCSLIDENode->FindCamera(piterIDs, plistCSLIDEInstance);
}

CSLIDELight *CSLIDEInstance::FindLight(CListIterInt *piterIDs, 
				       CListInt *plistCSLIDEInstance)
{
  ASSERT( piterIDs != NULL );
  ASSERT( plistCSLIDEInstance != NULL );

  return m_pCSLIDENode->FindLight(piterIDs, plistCSLIDEInstance);
}

////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: Fill these in
// 
VOID CSLIDEInstance::Preprocess()
{
}

VOID CSLIDEInstance::CheckDynamic()
{
}

VOID CSLIDEInstance::PreprocessGraph()
{
}

VOID CSLIDEInstance::Update()
{
}

VOID CSLIDEInstance::UpdateState()
{
}

VOID CSLIDEInstance::UpdateGraph()
{
}

VOID CSLIDEInstance::InsertIntoBound(CBound &bound)
{
}

VOID CSLIDEInstance::CalculateBound()
{
  // If this was called it means that the group above it
  // in the scene graph must recompute its bounding box
}

VOID CSLIDEInstance::Render(const Matrix &mGvp_proj, 
			    const Matrix &mGproj_vrc, 
			    const Matrix &mGvrc_proj, 
			    const Matrix &mGvrc_obj, 
			    const Matrix &mGobj_vrc, 
			    CBound *pCBoundClip, 
			    CListInt *plistLights,
			    CSLIDESurface *pCSLIDESurface,
			    SLIDEShadingType eShading, 
			    SLIDELODType eLOD)
{
  // You should be applying your transform to the mGvrc_obj matrix.
  // Think of VRC as the coordinate system of the root of the scene graph
  // for this assignment.
}

VOID CSLIDEInstance::ModifySurfacePointer(CSLIDESurface* &pCSLIDESurface)
{
}
// 
// END  : ASSIGNMENT 3: Fill these in
////////////////////////////////////////////////////////////////////////

VOID CSLIDEInstance::ModifyShadingFlag(SLIDEShadingType &/*eShading*/)
{
  // Ignore shading flags
}

VOID CSLIDEInstance::ModifyLODFlag(SLIDELODType &/*eLOD*/)
{
  // Ignore LOD flags
}

////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: Fill these in
// 
VOID CSLIDEInstance::RenderGeometry(const Matrix &mGvp_proj, 
				    const Matrix &mGproj_vrc, 
				    const Matrix &mGvrc_proj, 
				    const Matrix &mGvrc_obj, 
				    const Matrix &mGobj_vrc, 
				    CBound *pCBoundClip, 
				    CListInt *plistLights,
				    CSLIDESurface *pCSLIDESurface,
				    SLIDEShadingType eShading, 
				    SLIDELODType eLOD)
{
}

VOID CSLIDEInstance::RenderBound(const Matrix &mGvp_proj, 
				 const Matrix &mGproj_vrc, 
				 const Matrix &mGvrc_obj)
{
  // We do not store a bound at the instance level, so we must render
  // its child's bound transformed
}
// 
// END  : ASSIGNMENT 3: Fill these in
////////////////////////////////////////////////////////////////////////

VOID CSLIDEInstance::WriteSLIDE(FILE *fpOutputFile)
{
  CListIterInt iter;
  CSLIDETransform *pCSLIDETransform;

  ASSERT( fpOutputFile != NULL );

  fprintf(fpOutputFile, "  %s %s\n", 
	  gcppcSLIDEReservedWords[sliderwtInstance], 
	  m_pcChildID);

  if ( m_pcID != NULL )
    {
      fprintf(fpOutputFile, "    %s %s\n",
	      gcppcSLIDEReservedWords[sliderwtID], 
	      m_pcID);
    } 
  if ( m_pcSurfaceID != gcppcSLIDEReservedWords[sliderwtSLF_INHERIT] )
    {
      fprintf(fpOutputFile, "    %s %s\n",
	      gcppcSLIDEReservedWords[sliderwtSurface], 
	      m_pcSurfaceID);
    } 
  if ( m_dyniShading.GetValue() != slideshtSLF_INHERIT )
    {
      fprintf(fpOutputFile, "    %s %s\n",
	      gcppcSLIDEReservedWords[sliderwtShading], 
	      gcppcSLIDEShadingReservedWords[m_dyniShading.GetValue()]);
    }
  if ( m_dyniLOD.GetValue() != slidelodtSLF_FULL )
    {
      fprintf(fpOutputFile, "    %s %s\n",
	      gcppcSLIDEReservedWords[sliderwtLOD], 
	      gcppcSLIDELODReservedWords[m_dyniLOD.GetValue()]);
    }

  iter.Init(&m_listCSLIDETransform);

  for ( pCSLIDETransform = (CSLIDETransform*)iter.PeekFirst(); 
	pCSLIDETransform != NULL;
	pCSLIDETransform = (CSLIDETransform*)iter.PeekNext() )
    {
      pCSLIDETransform->WriteSLIDE(fpOutputFile);      
    }

  fprintf(fpOutputFile, "  %s\n", 
	  gcppcSLIDEReservedWords[sliderwtEndInstance]);
}

UINT_32 CSLIDEInstance::GetTransformCount()
{
  return m_listCSLIDETransform.GetLength();
}

//////////////////////////////////////////////////////////////////////////////
// CSLIDEInstanceIter Class
//

CSLIDEInstanceIter::CSLIDEInstanceIter()
{
}

CSLIDEInstanceIter::~CSLIDEInstanceIter()
{
}

VOID CSLIDEInstanceIter::Init(CSLIDEInstance *pCSLIDEInstance)
{
  CSLIDEPrimitiveIter::Init(pCSLIDEInstance);

  m_iter.Init(&(pCSLIDEInstance->m_listCSLIDETransform));
}

VOID CSLIDEInstanceIter::Uninit()
{
  CSLIDEPrimitiveIter::Uninit();
}

CSLIDETransform *CSLIDEInstanceIter::PeekFirstCSLIDETransform()
{
  return (CSLIDETransform*)(m_iter.PeekFirst());
}

CSLIDETransform *CSLIDEInstanceIter::PeekNextCSLIDETransform()
{
  return (CSLIDETransform*)(m_iter.PeekNext());
}
